@@ -1,24 +1,20 @@
 /*---------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2015, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  *---------------------------------------------------------------
  * Example problem:
- * 
- * The following test simulates a brusselator problem from chemical 
- * kinetics.  This is n PDE system with 3 components, Y = [u,v,w], 
+ *
+ * The following test simulates a brusselator problem from chemical
+ * kinetics.  This is n PDE system with 3 components, Y = [u,v,w],
  * satisfying the equations,
  *    u_t = du*u_xx + a - (w+1)*u + v*u^2
  *    v_t = dv*v_xx + w*u - v*u^2
@@ -27,27 +23,27 @@
  *    u(0,x) =  a  + 0.1*sin(pi*x)
  *    v(0,x) = b/a + 0.1*sin(pi*x)
  *    w(0,x) =  b  + 0.1*sin(pi*x),
- * and with stationary boundary conditions, i.e. 
+ * and with stationary boundary conditions, i.e.
  *    u_t(t,0) = u_t(t,1) = 0,
  *    v_t(t,0) = v_t(t,1) = 0,
  *    w_t(t,0) = w_t(t,1) = 0.
- * Note: these can also be implemented as Dirichlet boundary 
+ * Note: these can also be implemented as Dirichlet boundary
  * conditions with values identical to the initial conditions.
- * 
- * The spatial derivatives are computed using second-order 
- * centered differences, with the data distributed over N points 
+ *
+ * The spatial derivatives are computed using second-order
+ * centered differences, with the data distributed over N points
  * on a uniform spatial grid.
  *
- * The number of spatial points N, the parameters a, b, du, dv, 
- * dw and ep, as well as the desired relative and absolute solver 
- * tolerances, are provided in the input file 
+ * The number of spatial points N, the parameters a, b, du, dv,
+ * dw and ep, as well as the desired relative and absolute solver
+ * tolerances, are provided in the input file
  * input_brusselator1D.txt.
- * 
- * This program solves the problem with the DIRK method, using a
- * Newton iteration.  The inner linear systems are solved using 
- * the ARKKLU linear solver.
  *
- * 100 outputs are printed at equal intervals, and run statistics 
+ * This program solves the problem with the DIRK method, using a
+ * Newton iteration.  The inner linear systems are solved using
+ * the SUNLinSol_KLU linear solver.
+ *
+ * 100 outputs are printed at equal intervals, and run statistics
  * are printed at the end.
  *---------------------------------------------------------------*/
 
@@ -55,10 +51,22 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include <arkode/arkode.h>            /* prototypes for ARKode fcts., consts. */
-#include <arkode/arkode_klu.h>        /* prototype for ARKKLU solver */
-#include <nvector/nvector_serial.h>   /* serial N_Vector types, fcts., macros */
-#include <sundials/sundials_types.h>  /* def. of type 'realtype' */
+#include <arkode/arkode_arkstep.h>       /* prototypes for ARKStep fcts., consts */
+#include <nvector/nvector_serial.h>      /* serial N_Vector types, fcts., macros */
+#include <sunmatrix/sunmatrix_sparse.h>  /* access to sparse SUNMatrix           */
+#include <sunlinsol/sunlinsol_klu.h>     /* access to KLU SUNLinearSolver        */
+#include <sundials/sundials_types.h>     /* defs. of realtype, sunindextype, etc */
+#include <sundials/sundials_math.h>      /* def. of SUNRsqrt, etc.               */
+
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define GSYM "Lg"
+#define ESYM "Le"
+#define FSYM "Lf"
+#else
+#define GSYM "g"
+#define ESYM "e"
+#define FSYM "f"
+#endif
 
 /* accessor macros between (x,v) location and 1D NVector array */
 #define IDX(x,v) (3*(x)+v)
@@ -68,31 +76,30 @@
 #define TWO (RCONST(2.0))
 
 /* user data structure */
-typedef struct {  
-  int N;         /* number of intervals     */
-  realtype dx;   /* mesh spacing            */
-  realtype a;    /* constant forcing on u   */
-  realtype b;    /* steady-state value of w */
-  realtype du;   /* diffusion coeff for u   */
-  realtype dv;   /* diffusion coeff for v   */
-  realtype dw;   /* diffusion coeff for w   */
-  realtype ep;   /* stiffness parameter     */
-  SlsMat R;      /* temporary storage       */
+typedef struct {
+  sunindextype N;  /* number of intervals     */
+  realtype dx;     /* mesh spacing            */
+  realtype a;      /* constant forcing on u   */
+  realtype b;      /* steady-state value of w */
+  realtype du;     /* diffusion coeff for u   */
+  realtype dv;     /* diffusion coeff for v   */
+  realtype dw;     /* diffusion coeff for w   */
+  realtype ep;     /* stiffness parameter     */
+  SUNMatrix R;     /* temporary storage       */
 } *UserData;
 
 
 /* User-supplied Functions Called by the Solver */
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
-static int Jac(realtype t, N_Vector y, N_Vector fy, 
-	       SlsMat J, void *user_data, 
-	       N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
+static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J,
+               void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 /* Private function to check function return values */
-static int LaplaceMatrix(SlsMat Jac, UserData udata);
-static int ReactionJac(N_Vector y, SlsMat Jac, UserData udata);
+static int LaplaceMatrix(SUNMatrix Jac, UserData udata);
+static int ReactionJac(N_Vector y, SUNMatrix Jac, UserData udata);
 
 /* Private function to check function return values */
-static int check_flag(void *flagvalue, char *funcname, int opt);
+static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 
 /* Main Program */
@@ -105,7 +112,7 @@ int main()
   int Nvar = 3;
   UserData udata = NULL;
   realtype *data;
-  int N = 201;                  /* spatial mesh size */
+  sunindextype N = 201;         /* spatial mesh size */
   realtype a = 0.6;             /* problem parameters */
   realtype b = 2.0;
   realtype du = 0.025;
@@ -114,8 +121,7 @@ int main()
   realtype ep = 1.0e-5;         /* stiffness parameter */
   realtype reltol = 1.0e-6;     /* tolerances */
   realtype abstol = 1.0e-10;
-  int i;
-  long int NEQ, NNZ;
+  sunindextype NEQ, i, NNZ;
 
   /* general problem variables */
   int flag;                     /* reusable error-checking flag */
@@ -123,6 +129,8 @@ int main()
   N_Vector umask = NULL;
   N_Vector vmask = NULL;
   N_Vector wmask = NULL;
+  SUNMatrix A = NULL;           /* empty matrix object for solver */
+  SUNLinearSolver LS = NULL;    /* empty linear solver object */
   void *arkode_mem = NULL;
   realtype pi;
   FILE *FID, *UFID, *VFID, *WFID;
@@ -152,12 +160,12 @@ int main()
 
   /* Initial problem output */
   printf("\n1D Brusselator PDE test problem (KLU solver):\n");
-  printf("    N = %i,  NEQ = %li\n", udata->N, NEQ);
-  printf("    problem parameters:  a = %g,  b = %g,  ep = %g\n",
-	 udata->a, udata->b, udata->ep);
-  printf("    diffusion coefficients:  du = %g,  dv = %g,  dw = %g\n", 
-	 udata->du, udata->dv, udata->dw);
-  printf("    reltol = %.1e,  abstol = %.1e\n\n", reltol, abstol);
+  printf("    N = %li,  NEQ = %li\n", (long int) udata->N, (long int) NEQ);
+  printf("    problem parameters:  a = %"GSYM",  b = %"GSYM",  ep = %"GSYM"\n",
+         udata->a, udata->b, udata->ep);
+  printf("    diffusion coefficients:  du = %"GSYM",  dv = %"GSYM",  dw = %"GSYM"\n",
+         udata->du, udata->dv, udata->dw);
+  printf("    reltol = %.1"ESYM",  abstol = %.1"ESYM"\n\n", reltol, abstol);
 
   /* Initialize data structures */
   y = N_VNew_Serial(NEQ);           /* Create serial vector for solution */
@@ -197,33 +205,35 @@ int main()
   for (i=0; i<N; i++)  data[IDX(i,2)] = ONE;
 
 
-  /* Create the solver memory */
-  arkode_mem = ARKodeCreate();
-  if (check_flag((void *) arkode_mem, "ARKodeCreate", 0)) return 1;
-  
-  /* Call ARKodeInit to initialize the integrator memory and specify the
-     hand-side side function in y'=f(t,y), the inital time T0, and
-     the initial dependent variable vector y.  Note: since this
+  /* Call ARKStepCreate to initialize the ARK timestepper module and
+     specify the right-hand side function in y'=f(t,y), the inital time
+     T0, and the initial dependent variable vector y.  Note: since this
      problem is fully implicit, we set f_E to NULL and f_I to f. */
-  flag = ARKodeInit(arkode_mem, NULL, f, T0, y);
-  if (check_flag(&flag, "ARKodeInit", 1)) return 1;
+  arkode_mem = ARKStepCreate(NULL, f, T0, y);
+  if (check_flag((void *) arkode_mem, "ARKStepCreate", 0)) return 1;
 
   /* Set routines */
-  flag = ARKodeSetUserData(arkode_mem, (void *) udata);     /* Pass udata to user functions */
-  if (check_flag(&flag, "ARKodeSetUserData", 1)) return 1;
-  flag = ARKodeSStolerances(arkode_mem, reltol, abstol);    /* Specify tolerances */
-  if (check_flag(&flag, "ARKodeSStolerances", 1)) return 1;
+  flag = ARKStepSetUserData(arkode_mem, (void *) udata);     /* Pass udata to user functions */
+  if (check_flag(&flag, "ARKStepSetUserData", 1)) return 1;
+  flag = ARKStepSStolerances(arkode_mem, reltol, abstol);    /* Specify tolerances */
+  if (check_flag(&flag, "ARKStepSStolerances", 1)) return 1;
 
-  /* Specify the KLU sparse linear solver and Jacobian function */
+  /* Initialize sparse matrix data structure and KLU solver */
   NNZ = 5*NEQ;
-  flag = ARKKLU(arkode_mem, NEQ, NNZ);
-  if (check_flag(&flag, "ARKKLU", 1)) return 1;
-  flag = ARKSlsSetSparseJacFn(arkode_mem, Jac);
-  if (check_flag(&flag, "ARKSlsSetSparseJacFn", 1)) return 1;
- 
+  A = SUNSparseMatrix(NEQ, NEQ, NNZ, CSC_MAT);
+  if (check_flag((void *)A, "SUNSparseMatrix", 0)) return 1;
+  LS = SUNLinSol_KLU(y, A);
+  if (check_flag((void *)LS, "SUNLinSol_KLU", 0)) return 1;
+
+  /* Attach the matrix, linear solver, and Jacobian construction routine to ARKStep */
+  flag = ARKStepSetLinearSolver(arkode_mem, LS, A);        /* Attach matrix and LS */
+  if (check_flag(&flag, "ARKStepSetLinearSolver", 1)) return 1;
+  flag = ARKStepSetJacFn(arkode_mem, Jac);                 /* Supply Jac routine */
+  if (check_flag(&flag, "ARKStepSetJacFn", 1)) return 1;
+
    /* output spatial mesh to disk */
   FID = fopen("bruss_mesh.txt","w");
-  for (i=0; i<N; i++)  fprintf(FID,"  %.16e\n", udata->dx*i);
+  for (i=0; i<N; i++)  fprintf(FID,"  %.16"ESYM"\n", udata->dx*i);
   fclose(FID);
 
   /* Open output stream for results, access data arrays */
@@ -234,14 +244,14 @@ int main()
   if (check_flag((void *) data, "N_VGetArrayPointer", 0)) return 1;
 
   /* output initial condition to disk */
-  for (i=0; i<N; i++)  fprintf(UFID," %.16e", data[IDX(i,0)]);
-  for (i=0; i<N; i++)  fprintf(VFID," %.16e", data[IDX(i,1)]);
-  for (i=0; i<N; i++)  fprintf(WFID," %.16e", data[IDX(i,2)]);
+  for (i=0; i<N; i++)  fprintf(UFID," %.16"ESYM"", data[IDX(i,0)]);
+  for (i=0; i<N; i++)  fprintf(VFID," %.16"ESYM"", data[IDX(i,1)]);
+  for (i=0; i<N; i++)  fprintf(WFID," %.16"ESYM"", data[IDX(i,2)]);
   fprintf(UFID,"\n");
   fprintf(VFID,"\n");
   fprintf(WFID,"\n");
 
-  /* Main time-stepping loop: calls ARKode to perform the integration, then
+  /* Main time-stepping loop: calls ARKStepEvolve to perform the integration, then
      prints results.  Stops when the final time has been reached */
   t  = T0;
   dTout = Tf/Nt;
@@ -250,14 +260,14 @@ int main()
   printf("   ----------------------------------------------\n");
   for (iout=0; iout<Nt; iout++) {
 
-    flag = ARKode(arkode_mem, tout, y, &t, ARK_NORMAL);    /* call integrator */
+    flag = ARKStepEvolve(arkode_mem, tout, y, &t, ARK_NORMAL);    /* call integrator */
     u = N_VWL2Norm(y,umask);
-    u = sqrt(u*u/N);
+    u = SUNRsqrt(u*u/N);
     v = N_VWL2Norm(y,vmask);
-    v = sqrt(v*v/N);
+    v = SUNRsqrt(v*v/N);
     w = N_VWL2Norm(y,wmask);
-    w = sqrt(w*w/N);
-    printf("  %10.6f  %10.6f  %10.6f  %10.6f\n", t, u, v, w);
+    w = SUNRsqrt(w*w/N);
+    printf("  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"\n", t, u, v, w);
     if (flag >= 0) {                                       /* successful solve: update output time */
       tout += dTout;
       tout = (tout > Tf) ? Tf : tout;
@@ -267,9 +277,9 @@ int main()
     }
 
     /* output results to disk */
-    for (i=0; i<N; i++)  fprintf(UFID," %.16e", data[IDX(i,0)]);
-    for (i=0; i<N; i++)  fprintf(VFID," %.16e", data[IDX(i,1)]);
-    for (i=0; i<N; i++)  fprintf(WFID," %.16e", data[IDX(i,2)]);
+    for (i=0; i<N; i++)  fprintf(UFID," %.16"ESYM"", data[IDX(i,0)]);
+    for (i=0; i<N; i++)  fprintf(VFID," %.16"ESYM"", data[IDX(i,1)]);
+    for (i=0; i<N; i++)  fprintf(WFID," %.16"ESYM"", data[IDX(i,2)]);
     fprintf(UFID,"\n");
     fprintf(VFID,"\n");
     fprintf(WFID,"\n");
@@ -278,25 +288,25 @@ int main()
   fclose(UFID);
   fclose(VFID);
   fclose(WFID);
-    
+
 
   /* Print some final statistics */
-  flag = ARKodeGetNumSteps(arkode_mem, &nst);
-  check_flag(&flag, "ARKodeGetNumSteps", 1);
-  flag = ARKodeGetNumStepAttempts(arkode_mem, &nst_a);
-  check_flag(&flag, "ARKodeGetNumStepAttempts", 1);
-  flag = ARKodeGetNumRhsEvals(arkode_mem, &nfe, &nfi);
-  check_flag(&flag, "ARKodeGetNumRhsEvals", 1);
-  flag = ARKodeGetNumLinSolvSetups(arkode_mem, &nsetups);
-  check_flag(&flag, "ARKodeGetNumLinSolvSetups", 1);
-  flag = ARKodeGetNumErrTestFails(arkode_mem, &netf);
-  check_flag(&flag, "ARKodeGetNumErrTestFails", 1);
-  flag = ARKodeGetNumNonlinSolvIters(arkode_mem, &nni);
-  check_flag(&flag, "ARKodeGetNumNonlinSolvIters", 1);
-  flag = ARKodeGetNumNonlinSolvConvFails(arkode_mem, &ncfn);
-  check_flag(&flag, "ARKodeGetNumNonlinSolvConvFails", 1);
-  flag = ARKSlsGetNumJacEvals(arkode_mem, &nje);
-  check_flag(&flag, "ARKSlsGetNumJacEvals", 1);
+  flag = ARKStepGetNumSteps(arkode_mem, &nst);
+  check_flag(&flag, "ARKStepGetNumSteps", 1);
+  flag = ARKStepGetNumStepAttempts(arkode_mem, &nst_a);
+  check_flag(&flag, "ARKStepGetNumStepAttempts", 1);
+  flag = ARKStepGetNumRhsEvals(arkode_mem, &nfe, &nfi);
+  check_flag(&flag, "ARKStepGetNumRhsEvals", 1);
+  flag = ARKStepGetNumLinSolvSetups(arkode_mem, &nsetups);
+  check_flag(&flag, "ARKStepGetNumLinSolvSetups", 1);
+  flag = ARKStepGetNumErrTestFails(arkode_mem, &netf);
+  check_flag(&flag, "ARKStepGetNumErrTestFails", 1);
+  flag = ARKStepGetNumNonlinSolvIters(arkode_mem, &nni);
+  check_flag(&flag, "ARKStepGetNumNonlinSolvIters", 1);
+  flag = ARKStepGetNumNonlinSolvConvFails(arkode_mem, &ncfn);
+  check_flag(&flag, "ARKStepGetNumNonlinSolvConvFails", 1);
+  flag = ARKStepGetNumJacEvals(arkode_mem, &nje);
+  check_flag(&flag, "ARKStepGetNumJacEvals", 1);
 
   printf("\nFinal Solver Statistics:\n");
   printf("   Internal solver steps = %li (attempted = %li)\n", nst, nst_a);
@@ -308,13 +318,15 @@ int main()
   printf("   Total number of error test failures = %li\n", netf);
 
   /* Clean up and return with successful completion */
-  N_VDestroy_Serial(y);         /* Free vectors */
-  N_VDestroy_Serial(umask);
-  N_VDestroy_Serial(vmask);
-  N_VDestroy_Serial(wmask);
-  DestroySparseMat(udata->R);   /* Free user data */
+  N_VDestroy(y);                /* Free vectors */
+  N_VDestroy(umask);
+  N_VDestroy(vmask);
+  N_VDestroy(wmask);
+  SUNMatDestroy(udata->R);      /* Free user data */
   free(udata);
-  ARKodeFree(&arkode_mem);
+  ARKStepFree(&arkode_mem);     /* Free integrator memory */
+  SUNLinSolFree(LS);            /* Free linear solver */
+  SUNMatDestroy(A);             /* Free A matrix */
   return 0;
 }
 
@@ -326,17 +338,17 @@ int main()
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 {
   UserData udata = (UserData) user_data;      /* access problem data */
-  int N       = udata->N;                     /* set variable shortcuts */
-  realtype a  = udata->a;
-  realtype b  = udata->b;
-  realtype ep = udata->ep;
-  realtype du = udata->du;
-  realtype dv = udata->dv;
-  realtype dw = udata->dw;
-  realtype dx = udata->dx;
+  sunindextype N = udata->N;                     /* set variable shortcuts */
+  realtype a     = udata->a;
+  realtype b     = udata->b;
+  realtype ep    = udata->ep;
+  realtype du    = udata->du;
+  realtype dv    = udata->dv;
+  realtype dw    = udata->dw;
+  realtype dx    = udata->dx;
   realtype *Ydata=NULL, *dYdata=NULL;
   realtype uconst, vconst, wconst, u, ul, ur, v, vl, vr, w, wl, wr;
-  int i;
+  sunindextype i;
 
   Ydata = N_VGetArrayPointer(y);     /* access data arrays */
   if (check_flag((void *) Ydata, "N_VGetArrayPointer", 0)) return 1;
@@ -375,43 +387,45 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 
 
 /* Jacobian routine to compute J(t,y) = df/dy. */
-static int Jac(realtype t, N_Vector y, N_Vector fy, 
-	       SlsMat J, void *user_data, 
-	       N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
+static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J,
+               void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   /* problem data */
   UserData udata = (UserData) user_data;
-  int N = udata->N;
+  sunindextype N = udata->N;
 
   /* ensure that Jac is the correct size */
-  if ((J->M != N*3) || (J->N != N*3)) {
+  if ( (SUNSparseMatrix_Rows(J) != N*3) ||
+       (SUNSparseMatrix_Columns(J) != N*3) ) {
     printf("Jacobian calculation error: matrix is the wrong size!\n");
     return 1;
   }
-  
+
   /* Fill in the Laplace matrix */
   if (LaplaceMatrix(J, udata)) {
     printf("Jacobian calculation error in calling LaplaceMatrix!\n");
     return 1;
   }
 
-  /* Add in the Jacobian of the reaction terms matrix */
+  /* Create matrix for Jacobian of the reaction terms if necessary */
   if (udata->R == NULL) {
-    udata->R = NewSparseMat(J->M, J->N, J->NNZ);
+    udata->R = SUNSparseMatrix(SUNSparseMatrix_Rows(J),
+                               SUNSparseMatrix_Columns(J),
+                               SUNSparseMatrix_NNZ(J), CSC_MAT);
     if (udata->R == NULL) {
       printf("Jacobian calculation error in allocating R matrix!\n");
       return 1;
     }
   }
-      
-  /* Add in the Jacobian of the reaction terms matrix */
+
+  /* Compute the Jacobian of the reaction terms */
   if (ReactionJac(y, udata->R, udata)) {
     printf("Jacobian calculation error in calling ReactionJac!\n");
     return 1;
   }
 
   /* Add R to J */
-  if (SlsAddMat(J,udata->R) != 0) {
+  if (SUNMatScaleAdd(ONE, J, udata->R) != 0) {
     printf("Jacobian calculation error in adding sparse matrices!\n");
     return 1;
   }
@@ -427,20 +441,23 @@ static int Jac(realtype t, N_Vector y, N_Vector fy,
  *-------------------------------*/
 
 /* Routine to compute the stiffness matrix from (L*y) */
-static int LaplaceMatrix(SlsMat Lap, UserData udata)
+static int LaplaceMatrix(SUNMatrix Lap, UserData udata)
 {
-  int N = udata->N;  /* set shortcuts */
-  int i, nz=0;
+  sunindextype N = udata->N;  /* set shortcuts */
+  sunindextype i, nz=0;
   realtype uconst, uconst2, vconst, vconst2, wconst, wconst2;
+  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(Lap);
+  sunindextype *rowvals = SUNSparseMatrix_IndexValues(Lap);
+  realtype *data = SUNSparseMatrix_Data(Lap);
 
   /* clear out matrix */
-  SlsSetToZero(Lap);
+  SUNMatZero(Lap);
 
   /* set first column to zero */
-  Lap->colptrs[IDX(0,0)] = nz;
-  Lap->colptrs[IDX(0,1)] = nz;
-  Lap->colptrs[IDX(0,2)] = nz;
-  
+  colptrs[IDX(0,0)] = nz;
+  colptrs[IDX(0,1)] = nz;
+  colptrs[IDX(0,2)] = nz;
+
   /* iterate over nodes, filling in Laplacian entries depending on these */
   uconst  = (udata->du)/(udata->dx)/(udata->dx);
   uconst2 = -TWO*uconst;
@@ -451,59 +468,59 @@ static int LaplaceMatrix(SlsMat Lap, UserData udata)
   for (i=1; i<N-1; i++) {
 
     /* dependence on u at this node */
-    Lap->colptrs[IDX(i,0)] = nz;
+    colptrs[IDX(i,0)] = nz;
     if (i>1) {                /* node to left */
-      Lap->data[nz] = uconst;
-      Lap->rowvals[nz++] = IDX(i-1,0);
+      data[nz] = uconst;
+      rowvals[nz++] = IDX(i-1,0);
     }
 
-    Lap->data[nz] = uconst2;  /* self */
-    Lap->rowvals[nz++] = IDX(i,0);
+    data[nz] = uconst2;  /* self */
+    rowvals[nz++] = IDX(i,0);
 
     if (i<N-2) {              /* node to right */
-      Lap->data[nz] = uconst;
-      Lap->rowvals[nz++] = IDX(i+1,0);
+      data[nz] = uconst;
+      rowvals[nz++] = IDX(i+1,0);
     }
 
     /* dependence on v at this node */
-    Lap->colptrs[IDX(i,1)] = nz;
+    colptrs[IDX(i,1)] = nz;
     if (i>1) {                /* node to left */
-      Lap->data[nz] = vconst;
-      Lap->rowvals[nz++] = IDX(i-1,1);
+      data[nz] = vconst;
+      rowvals[nz++] = IDX(i-1,1);
     }
 
-    Lap->data[nz] = vconst2;  /* self */
-    Lap->rowvals[nz++] = IDX(i,1);
+    data[nz] = vconst2;  /* self */
+    rowvals[nz++] = IDX(i,1);
 
     if (i<N-2) {              /* node to right */
-      Lap->data[nz] = vconst;
-      Lap->rowvals[nz++] = IDX(i+1,1);
+      data[nz] = vconst;
+      rowvals[nz++] = IDX(i+1,1);
     }
 
     /* dependence on w at this node */
-    Lap->colptrs[IDX(i,2)] = nz;
+    colptrs[IDX(i,2)] = nz;
     if (i>1) {                /* node to left */
-      Lap->data[nz] = wconst;
-      Lap->rowvals[nz++] = IDX(i-1,2);
+      data[nz] = wconst;
+      rowvals[nz++] = IDX(i-1,2);
     }
 
-    Lap->data[nz] = wconst2;  /* self */
-    Lap->rowvals[nz++] = IDX(i,2);
+    data[nz] = wconst2;  /* self */
+    rowvals[nz++] = IDX(i,2);
 
     if (i<N-2) {              /* node to right */
-      Lap->data[nz] = wconst;
-      Lap->rowvals[nz++] = IDX(i+1,2);
+      data[nz] = wconst;
+      rowvals[nz++] = IDX(i+1,2);
     }
 
   }
 
   /* set last column to zero */
-  Lap->colptrs[IDX(N-1,0)] = nz;
-  Lap->colptrs[IDX(N-1,1)] = nz;
-  Lap->colptrs[IDX(N-1,2)] = nz;
-  
+  colptrs[IDX(N-1,0)] = nz;
+  colptrs[IDX(N-1,1)] = nz;
+  colptrs[IDX(N-1,2)] = nz;
+
   /* end of data */
-  Lap->colptrs[IDX(N-1,2)+1] = nz;
+  colptrs[IDX(N-1,2)+1] = nz;
 
   return 0;
 }
@@ -511,23 +528,26 @@ static int LaplaceMatrix(SlsMat Lap, UserData udata)
 
 
 /* Routine to compute the Jacobian matrix from R(y) */
-static int ReactionJac(N_Vector y, SlsMat Jac, UserData udata)
+static int ReactionJac(N_Vector y, SUNMatrix Jac, UserData udata)
 {
-  int N = udata->N;                            /* set shortcuts */
-  int i, nz=0;
+  sunindextype N = udata->N;                   /* set shortcuts */
+  sunindextype i, nz=0;
   realtype u, v, w;
   realtype ep = udata->ep;
+  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(Jac);
+  sunindextype *rowvals = SUNSparseMatrix_IndexValues(Jac);
+  realtype *data = SUNSparseMatrix_Data(Jac);
   realtype *Ydata = N_VGetArrayPointer(y);     /* access solution array */
   if (check_flag((void *) Ydata, "N_VGetArrayPointer", 0)) return 1;
 
   /* clear out matrix */
-  SlsSetToZero(Jac);
+  SUNMatZero(Jac);
 
   /* set first matrix column to zero */
-  Jac->colptrs[IDX(0,0)] = 0;
-  Jac->colptrs[IDX(0,1)] = 0;
-  Jac->colptrs[IDX(0,2)] = 0;
-  
+  colptrs[IDX(0,0)] = 0;
+  colptrs[IDX(0,1)] = 0;
+  colptrs[IDX(0,2)] = 0;
+
   /* iterate over interior nodes, filling in Jacobian entries */
   for (i=1; i<N-1; i++) {
 
@@ -537,47 +557,47 @@ static int ReactionJac(N_Vector y, SlsMat Jac, UserData udata)
     w = Ydata[IDX(i,2)];
 
     /* dependence on u at this node */
-    Jac->colptrs[IDX(i,0)] = nz;
+    colptrs[IDX(i,0)] = nz;
 
-    Jac->rowvals[nz] = IDX(i,0);        /* fu wrt u */
-    Jac->data[nz++] = TWO*u*v - w - ONE;
+    rowvals[nz] = IDX(i,0);        /* fu wrt u */
+    data[nz++] = TWO*u*v - w - ONE;
 
-    Jac->rowvals[nz] = IDX(i,1);        /* fv wrt u */
-    Jac->data[nz++] = w - TWO*u*v;
+    rowvals[nz] = IDX(i,1);        /* fv wrt u */
+    data[nz++] = w - TWO*u*v;
 
-    Jac->rowvals[nz] = IDX(i,2);        /* fw wrt u */
-    Jac->data[nz++] = -w;
+    rowvals[nz] = IDX(i,2);        /* fw wrt u */
+    data[nz++] = -w;
 
     /* dependence on v at this node */
-    Jac->colptrs[IDX(i,1)] = nz;
+    colptrs[IDX(i,1)] = nz;
 
-    Jac->rowvals[nz] = IDX(i,0);        /* fu wrt v */
-    Jac->data[nz++] = u*u;
+    rowvals[nz] = IDX(i,0);        /* fu wrt v */
+    data[nz++] = u*u;
 
-    Jac->rowvals[nz] = IDX(i,1);        /* fv wrt v */
-    Jac->data[nz++] = -u*u;
+    rowvals[nz] = IDX(i,1);        /* fv wrt v */
+    data[nz++] = -u*u;
 
     /* dependence on w at this node */
-    Jac->colptrs[IDX(i,2)] = nz;
+    colptrs[IDX(i,2)] = nz;
 
-    Jac->rowvals[nz] = IDX(i,0);        /* fu wrt w */
-    Jac->data[nz++] = -u;
+    rowvals[nz] = IDX(i,0);        /* fu wrt w */
+    data[nz++] = -u;
 
-    Jac->rowvals[nz] = IDX(i,1);        /* fv wrt w */
-    Jac->data[nz++] = u;
+    rowvals[nz] = IDX(i,1);        /* fv wrt w */
+    data[nz++] = u;
 
-    Jac->rowvals[nz] = IDX(i,2);        /* fw wrt w */
-    Jac->data[nz++] = -ONE/ep - u;
+    rowvals[nz] = IDX(i,2);        /* fw wrt w */
+    data[nz++] = -ONE/ep - u;
 
   }
 
   /* set last matrix column to zero */
-  Jac->colptrs[IDX(N-1,0)] = nz;
-  Jac->colptrs[IDX(N-1,1)] = nz;
-  Jac->colptrs[IDX(N-1,2)] = nz;
+  colptrs[IDX(N-1,0)] = nz;
+  colptrs[IDX(N-1,1)] = nz;
+  colptrs[IDX(N-1,2)] = nz;
 
   /* end of data */
-  Jac->colptrs[IDX(N-1,2)+1] = nz;
+  colptrs[IDX(N-1,2)+1] = nz;
 
   return 0;
 }
@@ -590,16 +610,16 @@ static int ReactionJac(N_Vector y, SlsMat Jac, UserData udata)
     opt == 1 means SUNDIALS function returns a flag so check if
              flag >= 0
     opt == 2 means function allocates memory so check if returned
-             NULL pointer  
+             NULL pointer
 */
-static int check_flag(void *flagvalue, char *funcname, int opt)
+static int check_flag(void *flagvalue, const char *funcname, int opt)
 {
   int *errflag;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && flagvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return 1; }
 
   /* Check if flag < 0 */
@@ -607,13 +627,13 @@ static int check_flag(void *flagvalue, char *funcname, int opt)
     errflag = (int *) flagvalue;
     if (*errflag < 0) {
       fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              funcname, *errflag);
       return 1; }}
 
   /* Check if function returned NULL pointer - no memory allocated */
   else if (opt == 2 && flagvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return 1; }
 
   return 0;
