@@ -1,10 +1,16 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 4272 $
- * $Date: 2014-12-02 11:19:41 -0800 (Tue, 02 Dec 2014) $
- * -----------------------------------------------------------------
+/* -----------------------------------------------------------------
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * This program solves a stiff ODE system that arises from a system
  * of partial differential equations. The PDE system is a food web
  * population model, with predator-prey interaction and diffusion on
@@ -46,7 +52,7 @@
  * mesh. The resulting ODE system is stiff.
  *
  * The ODE system is solved by CVODES using Newton iteration and
- * the CVSPGMR linear solver (scaled preconditioned GMRES).
+ * the SUNLinSol_SPGMR linear solver (scaled preconditioned GMRES).
  *
  * The preconditioner matrix used is the product of two matrices:
  * (1) A matrix, only defined implicitly, based on a fixed number
@@ -79,19 +85,18 @@
  * Matrix Methods in Stiff ODE Systems, J. Appl. Math. & Comp., 31
  * (1989), pp. 40-91.  Also available as Lawrence Livermore National
  * Laboratory Report UCRL-95088, Rev. 1, June 1987.
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
-#include <cvodes/cvodes.h>
-#include <cvodes/cvodes_spgmr.h>
-#include <nvector/nvector_serial.h>
-#include <sundials/sundials_dense.h>
-#include <sundials/sundials_types.h>
-#include <sundials/sundials_math.h>
+#include <cvodes/cvodes.h>             /* main integrator header file          */
+#include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
+#include <sunlinsol/sunlinsol_spgmr.h> /* access to SPGMR SUNLinearSolver      */
+#include <sundials/sundials_dense.h>   /* use generic dense solver in precond. */
+#include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
+#include <sundials/sundials_math.h>    /* contains the macros ABS, SUNSQR, EXP */
 
 #define ZERO RCONST(0.0)
 #define ONE  RCONST(1.0)
@@ -144,7 +149,7 @@
 
 typedef struct {
   realtype **P[NGRP];
-  long int *pivot[NGRP];
+  sunindextype *pivot[NGRP];
   int ns,  mxns, mp, mq, mx, my, ngrp, ngx, ngy, mxmp;
   int jgx[NGX+1], jgy[NGY+1], jigx[MX], jigy[MY];
   int jxr[NGX], jyr[NGY];
@@ -153,6 +158,7 @@ typedef struct {
   realtype fsave[NEQ];
   realtype fBsave[NEQ];
   N_Vector rewt;
+  N_Vector vtemp;
   void *cvode_mem;
   int indexB;
 } *WebData;
@@ -169,13 +175,12 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
 static int Precond(realtype t, N_Vector c, N_Vector fc,
                    booleantype jok, booleantype *jcurPtr, 
-                   realtype gamma, void *user_data,
-                   N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3);
+                   realtype gamma, void *user_data);
 
 static int PSolve(realtype t, N_Vector c, N_Vector fc,
                   N_Vector r, N_Vector z,
                   realtype gamma, realtype delta,
-                  int lr, void *user_data, N_Vector vtemp);
+                  int lr, void *user_data);
 
 static int fB(realtype t, N_Vector c, N_Vector cB, 
                N_Vector cBdot, void *user_data);
@@ -183,14 +188,13 @@ static int fB(realtype t, N_Vector c, N_Vector cB,
 static int PrecondB(realtype t, N_Vector c, 
                     N_Vector cB, N_Vector fcB, booleantype jok, 
                     booleantype *jcurPtr, realtype gamma,
-                    void *user_data,
-                    N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3);
+                    void *user_data);
 
 static int PSolveB(realtype t, N_Vector c, 
                    N_Vector cB, N_Vector fcB, 
                    N_Vector r, N_Vector z,
                    realtype gamma, realtype delta, 
-                   int lr, void *user_data, N_Vector vtemp);
+                   int lr, void *user_data);
 
 /* Prototypes for private functions */
 
@@ -208,7 +212,7 @@ static void fblock (realtype t, realtype cdata[], int jx, int jy, realtype cdotd
                     WebData wdata);
 static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata);
 static realtype doubleIntgr(N_Vector c, int i, WebData wdata);
-static int check_flag(void *flagvalue, char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /* Small Vector Kernels */
 
@@ -230,8 +234,9 @@ int main(int argc, char *argv[])
   N_Vector c;
   WebData wdata;
   void *cvode_mem;
+  SUNLinearSolver LS, LSB;
 
-  int flag, ncheck;
+  int retval, ncheck;
   
   int indexB;
 
@@ -242,106 +247,121 @@ int main(int argc, char *argv[])
   cB = NULL;
   wdata = NULL;
   cvode_mem = NULL;
+  LS = LSB = NULL;
 
   /* Allocate and initialize user data */
 
   wdata = AllocUserData();
-  if(check_flag((void *)wdata, "AllocUserData", 2)) return(1);
+  if(check_retval((void *)wdata, "AllocUserData", 2)) return(1);
   InitUserData(wdata);
 
   /* Set-up forward problem */
 
   /* Initializations */
   c = N_VNew_Serial(NEQ+1);
-  if(check_flag((void *)c, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)c, "N_VNew_Serial", 0)) return(1);
   CInit(c, wdata);
 
   /* Call CVodeCreate/CVodeInit for forward run */
   printf("\nCreate and allocate CVODES memory for forward run\n");
-  cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
-  if(check_flag((void *)cvode_mem, "CVodeCreate", 0)) return(1);
+  cvode_mem = CVodeCreate(CV_BDF);
+  if(check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
   wdata->cvode_mem = cvode_mem; /* Used in Precond */
-  flag = CVodeSetUserData(cvode_mem, wdata);
-  if(check_flag(&flag, "CVodeSetUserData", 1)) return(1);
-  flag = CVodeInit(cvode_mem, f, T0, c);
-  if(check_flag(&flag, "CVodeInit", 1)) return(1);
-  flag = CVodeSStolerances(cvode_mem, reltol, abstol);
-  if(check_flag(&flag, "CVodeSStolerances", 1)) return(1);
+  retval = CVodeSetUserData(cvode_mem, wdata);
+  if(check_retval(&retval, "CVodeSetUserData", 1)) return(1);
+  retval = CVodeInit(cvode_mem, f, T0, c);
+  if(check_retval(&retval, "CVodeInit", 1)) return(1);
+  retval = CVodeSStolerances(cvode_mem, reltol, abstol);
+  if(check_retval(&retval, "CVodeSStolerances", 1)) return(1);
 
-  /* Call CVSpgmr for forward run */
-  flag = CVSpgmr(cvode_mem, PREC_LEFT, 0);
-  if(check_flag(&flag, "CVSpgmr", 1)) return(1);
-  flag = CVSpilsSetPreconditioner(cvode_mem, Precond, PSolve);
-  if(check_flag(&flag, "CVSpilsSetPreconditioner", 1)) return(1);
+  /* Create SUNLinSol_SPGMR linear solver for forward run */
+  LS = SUNLinSol_SPGMR(c, PREC_LEFT, 0);
+  if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
+
+  /* Attach the linear sovler */
+  retval = CVodeSetLinearSolver(cvode_mem, LS, NULL);
+  if (check_retval(&retval, "CVodeSetLinearSolver", 1)) return 1;
+
+  /* Set the preconditioner solve and setup functions */
+  retval = CVodeSetPreconditioner(cvode_mem, Precond, PSolve);
+  if(check_retval(&retval, "CVodeSetPreconditioner", 1)) return(1);
 
   /* Set-up adjoint calculations */
 
   printf("\nAllocate global memory\n");
-  flag = CVodeAdjInit(cvode_mem, NSTEPS, CV_HERMITE);
-  if(check_flag(&flag, "CVadjInit", 1)) return(1);
+  retval = CVodeAdjInit(cvode_mem, NSTEPS, CV_HERMITE);
+  if(check_retval(&retval, "CVadjInit", 1)) return(1);
 
   /* Perform forward run */
 
   printf("\nForward integration\n");
-  flag = CVodeF(cvode_mem, TOUT, c, &t, CV_NORMAL, &ncheck);
-  if(check_flag(&flag, "CVodeF", 1)) return(1);
+  retval = CVodeF(cvode_mem, TOUT, c, &t, CV_NORMAL, &ncheck);
+  if(check_retval(&retval, "CVodeF", 1)) return(1);
 
   printf("\nncheck = %d\n", ncheck);
 
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("\n   G = int_t int_x int_y c%d(t,x,y) dx dy dt = %Lf \n\n", 
-         ISPEC, NV_DATA_S(c)[NEQ]);
+         ISPEC, N_VGetArrayPointer(c)[NEQ]);
 #else
   printf("\n   G = int_t int_x int_y c%d(t,x,y) dx dy dt = %f \n\n", 
-         ISPEC, NV_DATA_S(c)[NEQ]);
+         ISPEC, N_VGetArrayPointer(c)[NEQ]);
 #endif
 
   /* Set-up backward problem */
 
   /* Allocate cB */
   cB = N_VNew_Serial(NEQ);
-  if(check_flag((void *)cB, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)cB, "N_VNew_Serial", 0)) return(1);
   /* Initialize cB = 0 */
   N_VConst(ZERO, cB);
 
   /* Create and allocate CVODES memory for backward run */
   printf("\nCreate and allocate CVODES memory for backward run\n");
-  flag = CVodeCreateB(cvode_mem, CV_BDF, CV_NEWTON, &indexB);
-  if(check_flag(&flag, "CVodeCreateB", 1)) return(1);
-  flag = CVodeSetUserDataB(cvode_mem, indexB, wdata);
-  if(check_flag(&flag, "CVodeSetUserDataB", 1)) return(1);
-  flag = CVodeSetMaxNumStepsB(cvode_mem, indexB, 1000);
-  if(check_flag(&flag, "CVodeSetMaxNumStepsB", 1)) return(1);
-  flag = CVodeInitB(cvode_mem, indexB, fB, TOUT, cB);
-  if(check_flag(&flag, "CVodeInitB", 1)) return(1);
-  flag = CVodeSStolerancesB(cvode_mem, indexB, reltolB, abstolB);
-  if(check_flag(&flag, "CVodeSStolerancesB", 1)) return(1);
+  retval = CVodeCreateB(cvode_mem, CV_BDF, &indexB);
+  if(check_retval(&retval, "CVodeCreateB", 1)) return(1);
+  retval = CVodeSetUserDataB(cvode_mem, indexB, wdata);
+  if(check_retval(&retval, "CVodeSetUserDataB", 1)) return(1);
+  retval = CVodeSetMaxNumStepsB(cvode_mem, indexB, 1000);
+  if(check_retval(&retval, "CVodeSetMaxNumStepsB", 1)) return(1);
+  retval = CVodeInitB(cvode_mem, indexB, fB, TOUT, cB);
+  if(check_retval(&retval, "CVodeInitB", 1)) return(1);
+  retval = CVodeSStolerancesB(cvode_mem, indexB, reltolB, abstolB);
+  if(check_retval(&retval, "CVodeSStolerancesB", 1)) return(1);
 
   wdata->indexB = indexB;
 
-  /* Call CVSpgmr */
-  flag = CVSpgmrB(cvode_mem, indexB, PREC_LEFT, 0);
-  if(check_flag(&flag, "CVSpgmrB", 1)) return(1);
-  flag = CVSpilsSetPreconditionerB(cvode_mem, indexB, PrecondB, PSolveB);
-  if(check_flag(&flag, "CVSpilsSetPreconditionerB", 1)) return(1);
+  /* Create SUNLinSol_SPGMR linear solver for backward run */
+  LSB = SUNLinSol_SPGMR(cB, PREC_LEFT, 0);
+  if(check_retval((void *)LSB, "SUNLinSol_SPGMR", 0)) return(1);
+
+  /* Attach the linear sovler */
+  retval = CVodeSetLinearSolverB(cvode_mem, indexB, LSB, NULL);
+  if (check_retval(&retval, "CVodeSetLinearSolverB", 1)) return 1;
+
+  /* Set the preconditioner solve and setup functions */
+  retval = CVodeSetPreconditionerB(cvode_mem, indexB, PrecondB, PSolveB);
+  if(check_retval(&retval, "CVodeSetPreconditionerB", 1)) return(1);
 
   /* Perform backward integration */
 
   printf("\nBackward integration\n");
-  flag = CVodeB(cvode_mem, T0, CV_NORMAL);
-  if(check_flag(&flag, "CVodeB", 1)) return(1);
+  retval = CVodeB(cvode_mem, T0, CV_NORMAL);
+  if(check_retval(&retval, "CVodeB", 1)) return(1);
 
-  flag = CVodeGetB(cvode_mem, indexB, &t, cB);
-  if(check_flag(&flag, "CVodeGetB", 1)) return(1);
+  retval = CVodeGetB(cvode_mem, indexB, &t, cB);
+  if(check_retval(&retval, "CVodeGetB", 1)) return(1);
 
   PrintOutput(cB, NS, MXNS, wdata);
 
   /* Free all memory */
   CVodeFree(&cvode_mem);
 
-  N_VDestroy_Serial(c);
-  N_VDestroy_Serial(cB);
+  N_VDestroy(c);
+  N_VDestroy(cB);
+  SUNLinSolFree(LS);
+  SUNLinSolFree(LSB);
 
   FreeUserData(wdata);
 
@@ -368,8 +388,8 @@ static int f(realtype t, N_Vector c, N_Vector cdot, void *user_data)
   WebData wdata;
   
   wdata = (WebData) user_data;
-  cdata = NV_DATA_S(c);
-  cdotdata = NV_DATA_S(cdot);
+  cdata = N_VGetArrayPointer(c);
+  cdotdata = N_VGetArrayPointer(cdot);
   
   mxns = wdata->mxns;
   ns = wdata->ns;
@@ -430,15 +450,14 @@ static int f(realtype t, N_Vector c, N_Vector cdot, void *user_data)
  
 static int Precond(realtype t, N_Vector c, N_Vector fc,
                    booleantype jok, booleantype *jcurPtr, 
-                   realtype gamma, void *user_data,
-                   N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3)
+                   realtype gamma, void *user_data)
 {
-  int N;
+  int N, retval;
   realtype ***P;
-  long int **pivot, ier;
+  sunindextype **pivot;
   int i, if0, if00, ig, igx, igy, j, jj, jx, jy;
-  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, flag;
-  long int mp;
+  int *jxr, *jyr, ngrp, ngx, ngy, mxmp;
+  sunindextype mp, denseretval;
   realtype uround, fac, r, r0, save, srur;
   realtype *f1, *fsave, *cdata, *rewtdata;
   void *cvode_mem;
@@ -448,11 +467,11 @@ static int Precond(realtype t, N_Vector c, N_Vector fc,
   wdata = (WebData) user_data;
   cvode_mem = wdata->cvode_mem;
   rewt = wdata->rewt;
-  flag = CVodeGetErrWeights(cvode_mem, rewt);
-  if(check_flag(&flag, "CVodeGetErrWeights", 1)) return(1);
+  retval = CVodeGetErrWeights(cvode_mem, rewt);
+  if(check_retval(&retval, "CVodeGetErrWeights", 1)) return(1);
 
-  cdata = NV_DATA_S(c);
-  rewtdata = NV_DATA_S(rewt);
+  cdata = N_VGetArrayPointer(c);
+  rewtdata = N_VGetArrayPointer(rewt);
 
   uround = UNIT_ROUNDOFF;
 
@@ -472,7 +491,7 @@ static int Precond(realtype t, N_Vector c, N_Vector fc,
      Here, fsave contains the base value of the rate vector and 
      r0 is a minimum increment factor for the difference quotient. */
 
-  f1 = NV_DATA_S(vtemp1);
+  f1 = N_VGetArrayPointer(wdata->vtemp);
 
   fac = N_VWrmsNorm (fc, rewt);
   N = NEQ+1;
@@ -507,11 +526,11 @@ static int Precond(realtype t, N_Vector c, N_Vector fc,
 
    for (ig = 0; ig < ngrp; ig++) {
      denseAddIdentity(P[ig], mp);
-     ier = denseGETRF(P[ig], mp, mp, pivot[ig]);
-     if (ier != 0) return(1);
+     denseretval = denseGETRF(P[ig], mp, mp, pivot[ig]);
+     if (denseretval != 0) return(1);
    }
 
-  *jcurPtr = TRUE;
+  *jcurPtr = SUNTRUE;
   return(0);
 }
 
@@ -529,12 +548,12 @@ static int Precond(realtype t, N_Vector c, N_Vector fc,
 static int PSolve(realtype t, N_Vector c, N_Vector fc,
                   N_Vector r, N_Vector z,
                   realtype gamma, realtype delta,
-                  int lr, void *user_data, N_Vector vtemp)
+                  int lr, void *user_data)
 {
   realtype ***P;
-  long int **pivot;
+  sunindextype **pivot;
   int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx;
-  long int mp;
+  sunindextype mp;
   WebData wdata;
 
   wdata = (WebData) user_data;
@@ -543,7 +562,7 @@ static int PSolve(realtype t, N_Vector c, N_Vector fc,
 
   /* call GSIter for Gauss-Seidel iterations */
 
-  GSIter(gamma, z, vtemp, wdata);
+  GSIter(gamma, z, wdata->vtemp, wdata);
 
   /* Do backsolves for inverse of block-diagonal preconditioner factor */
  
@@ -562,13 +581,13 @@ static int PSolve(realtype t, N_Vector c, N_Vector fc,
     for (jx = 0; jx < mx; jx++) {
       igx = jigx[jx];
       ig = igx + igy*ngx;
-      denseGETRS(P[ig], mp, pivot[ig], &(NV_DATA_S(z)[iv]));
+      denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
       iv += mp;
     }
   }
 
   /* Solve for the quadrature variable */
-  NV_DATA_S(z)[NEQ] = NV_DATA_S(r)[NEQ] + gamma*doubleIntgr(z,ISPEC,wdata);
+  N_VGetArrayPointer(z)[NEQ] = N_VGetArrayPointer(r)[NEQ] + gamma*doubleIntgr(z,ISPEC,wdata);
 
   return(0);
 }
@@ -591,9 +610,9 @@ static int fB(realtype t, N_Vector c, N_Vector cB,
   realtype gu[NS];
 
   wdata = (WebData) user_data;
-  cdata = NV_DATA_S(c);
-  cBdata = NV_DATA_S(cB);
-  cBdotdata = NV_DATA_S(cBdot);
+  cdata = N_VGetArrayPointer(c);
+  cBdata = N_VGetArrayPointer(cB);
+  cBdotdata = N_VGetArrayPointer(cBdot);
 
   mxns = wdata->mxns;
   ns = wdata->ns;
@@ -647,15 +666,14 @@ static int fB(realtype t, N_Vector c, N_Vector cB,
 static int PrecondB(realtype t, N_Vector c, 
                     N_Vector cB, N_Vector fcB, booleantype jok, 
                     booleantype *jcurPtr, realtype gamma,
-                    void *user_data,
-                    N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3)
+                    void *user_data)
 {
-  int N;
+  int N, retval;
   realtype ***P;
-  long int **pivot, ier;
+  sunindextype **pivot;
   int i, if0, if00, ig, igx, igy, j, jj, jx, jy;
-  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, flag;
-  long int mp;
+  int *jxr, *jyr, ngrp, ngx, ngy, mxmp;
+  sunindextype mp, denseretval;
   realtype uround, fac, r, r0, save, srur;
   realtype *f1, *fsave, *cdata, *rewtdata;
   void *cvode_mem;
@@ -664,13 +682,13 @@ static int PrecondB(realtype t, N_Vector c,
 
   wdata = (WebData) user_data;
   cvode_mem = CVodeGetAdjCVodeBmem(wdata->cvode_mem, wdata->indexB);
-  if(check_flag((void *)cvode_mem, "CVadjGetCVodeBmem", 0)) return(1);
+  if(check_retval((void *)cvode_mem, "CVadjGetCVodeBmem", 0)) return(1);
   rewt = wdata->rewt;
-  flag = CVodeGetErrWeights(cvode_mem, rewt);
-  if(check_flag(&flag, "CVodeGetErrWeights", 1)) return(1);
+  retval = CVodeGetErrWeights(cvode_mem, rewt);
+  if(check_retval(&retval, "CVodeGetErrWeights", 1)) return(1);
 
-  cdata = NV_DATA_S(c);
-  rewtdata = NV_DATA_S(rewt);
+  cdata = N_VGetArrayPointer(c);
+  rewtdata = N_VGetArrayPointer(rewt);
 
   uround = UNIT_ROUNDOFF;
 
@@ -690,7 +708,7 @@ static int PrecondB(realtype t, N_Vector c,
      Here, fsave contains the base value of the rate vector and 
      r0 is a minimum increment factor for the difference quotient. */
 
-  f1 = NV_DATA_S(vtemp1);
+  f1 = N_VGetArrayPointer(wdata->vtemp);
   fac = N_VWrmsNorm (fcB, rewt);
   N = NEQ;
   r0 = RCONST(1000.0)*SUNRabs(gamma)*uround*N*fac;
@@ -724,11 +742,11 @@ static int PrecondB(realtype t, N_Vector c,
 
    for (ig = 0; ig < ngrp; ig++) {
      denseAddIdentity(P[ig], mp);
-     ier = denseGETRF(P[ig], mp, mp, pivot[ig]);
-     if (ier != 0) return(1);
+     denseretval = denseGETRF(P[ig], mp, mp, pivot[ig]);
+     if (denseretval != 0) return(1);
    }
 
-  *jcurPtr = TRUE;
+  *jcurPtr = SUNTRUE;
   return(0);
 }
 
@@ -740,12 +758,12 @@ static int PSolveB(realtype t, N_Vector c,
                    N_Vector cB, N_Vector fcB, 
                    N_Vector r, N_Vector z,
                    realtype gamma, realtype delta, 
-                   int lr, void *user_data, N_Vector vtemp)
+                   int lr, void *user_data)
 {
   realtype ***P;
-  long int **pivot;
+  sunindextype **pivot;
   int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx;
-  long int mp;
+  sunindextype mp;
   WebData wdata;
 
   wdata = (WebData) user_data;
@@ -754,7 +772,7 @@ static int PSolveB(realtype t, N_Vector c,
 
   /* call GSIter for Gauss-Seidel iterations (same routine but with gamma=-gamma) */
 
-  GSIter(-gamma, z, vtemp, wdata);
+  GSIter(-gamma, z, wdata->vtemp, wdata);
 
   /* Do backsolves for inverse of block-diagonal preconditioner factor */
  
@@ -773,7 +791,7 @@ static int PSolveB(realtype t, N_Vector c,
     for (jx = 0; jx < mx; jx++) {
       igx = jigx[jx];
       ig = igx + igy*ngx;
-      denseGETRS(P[ig], mp, pivot[ig], &(NV_DATA_S(z)[iv]));
+      denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
       iv += mp;
     }
   }
@@ -799,9 +817,11 @@ static WebData AllocUserData(void)
   wdata = (WebData) malloc(sizeof *wdata);
   for(i=0; i < ngrp; i++) {
     (wdata->P)[i] = newDenseMat(ns, ns);
-    (wdata->pivot)[i] = newLintArray(ns);
+    (wdata->pivot)[i] = newIndexArray(ns);
   }
-  wdata->rewt = N_VNew_Serial(NEQ+1);
+  wdata->rewt  = N_VNew_Serial(NEQ+1);
+  wdata->vtemp = N_VNew_Serial(NEQ+1);
+
   return(wdata);
 }
 
@@ -899,7 +919,7 @@ static void CInit(N_Vector c, WebData wdata)
   int i, ici, ioff, iyoff, jx, jy, ns, mxns;
   realtype argx, argy, x, y, dx, dy, x_factor, y_factor, *cdata;
   
-  cdata = NV_DATA_S(c);
+  cdata = N_VGetArrayPointer(c);
   ns = wdata->ns;
   mxns = wdata->mxns;
   dx = wdata->dx;
@@ -1029,8 +1049,8 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x,
   realtype beta[NS], beta2[NS], cof1[NS], gam[NS], gam2[NS];
   realtype temp, *cox, *coy, *xd, *zd;
 
-  xd = NV_DATA_S(x);
-  zd = NV_DATA_S(z);
+  xd = N_VGetArrayPointer(x);
+  zd = N_VGetArrayPointer(z);
   ns = wdata->ns;
   mx = wdata->mx;
   my = wdata->my;
@@ -1211,7 +1231,7 @@ static void PrintOutput(N_Vector cB, int ns, int mxns, WebData wdata)
 
   x = y = ZERO;
 
-  cdata = NV_DATA_S(cB);
+  cdata = N_VGetArrayPointer(cB);
 
   for (i=1; i <= ns; i++) {
 
@@ -1261,7 +1281,7 @@ static realtype doubleIntgr(N_Vector c, int i, WebData wdata)
   realtype intgr_xy, intgr_x;
   int jx, jy;
 
-  cdata = NV_DATA_S(c);
+  cdata = N_VGetArrayPointer(c);
 
   ns   = wdata->ns;
   mx   = wdata->mx;
@@ -1321,7 +1341,8 @@ static void FreeUserData(WebData wdata)
     destroyMat((wdata->P)[i]);
     destroyArray((wdata->pivot)[i]);
   }
-  N_VDestroy_Serial(wdata->rewt);
+  N_VDestroy(wdata->rewt);
+  N_VDestroy(wdata->vtemp);
   free(wdata);
 }
 
@@ -1329,32 +1350,32 @@ static void FreeUserData(WebData wdata)
  * Check function return value.
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns an integer value so check if
+ *             retval < 0
  *    opt == 2 means function allocates memory so check if returned
  *             NULL pointer 
  */
 
-static int check_flag(void *flagvalue, char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && returnvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1); }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *retval);
       return(1); }}
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && returnvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1); }
