@@ -12,7 +12,7 @@
  * The nonlinear system is solved by KINSOL using the Picard
  * iteration and the SUNBAND linear solver.
  *
- * This file is strongly based on the kinLaplace_bnd.c file 
+ * This file is strongly based on the kinLaplace_bnd.c file
  * developed by Radu Serban.
  * -----------------------------------------------------------------
  */
@@ -24,7 +24,6 @@
 #include <kinsol/kinsol.h>             /* access to KINSOL func., consts. */
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector       */
 #include <sunmatrix/sunmatrix_band.h>  /* access to band SUNMatrix        */
-#include <kinsol/kinsol_direct.h>      /* access to KINDls interface      */
 #include <sunlinsol/sunlinsol_band.h>  /* access to band SUNLinearSolver  */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype */
 #include <sundials/sundials_math.h>    /* access to SUNRexp               */
@@ -45,11 +44,11 @@
 
 /* IJth is defined in order to isolate the translation from the
    mathematical 2-dimensional structure of the dependent variable vector
-   to the underlying 1-dimensional storage. 
+   to the underlying 1-dimensional storage.
    IJth(vdata,i,j) references the element in the vdata array for
    u at mesh point (i,j), where 1 <= i <= NX, 1 <= j <= NY.
    The vdata array is obtained via the call vdata = N_VGetArrayPointer_Serial(v),
-   where v is an N_Vector. 
+   where v is an N_Vector.
    The variables are ordered by the y index j, then by the x index i. */
 
 #define IJth(vdata,i,j) (vdata[(j-1) + (i-1)*NY])
@@ -86,7 +85,7 @@ int main()
   /* -------------------------
    * Print problem description
    * ------------------------- */
-  
+
   printf("\n2D elliptic PDE on unit square\n");
   printf("   d^2 u / dx^2 + d^2 u / dy^2 = u^3 - u + 2.0\n");
   printf(" + homogeneous Dirichlet boundary conditions\n\n");
@@ -120,12 +119,12 @@ int main()
   if (check_flag(&flag, "KINInit", 1)) return(1);
 
   /* -------------------
-   * Set optional inputs 
+   * Set optional inputs
    * ------------------- */
 
   /* Specify stopping tolerance based on residual */
 
-  fnormtol  = FTOL; 
+  fnormtol  = FTOL;
   flag = KINSetFuncNormTol(kmem, fnormtol);
   if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
 
@@ -133,39 +132,39 @@ int main()
    * Create band SUNMatrix
    * ------------------------- */
 
-  J = SUNBandMatrix(NEQ, NX, NX, 2*NX);
+  J = SUNBandMatrix(NEQ, NX, NX);
   if(check_flag((void *)J, "SUNBandMatrix", 0)) return(1);
 
   /* ---------------------------
    * Create band SUNLinearSolver
    * --------------------------- */
 
-  LS = SUNBandLinearSolver(y, J);
-  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Band(y, J);
+  if(check_flag((void *)LS, "SUNLinSol_Band", 0)) return(1);
 
   /* -------------------------
-   * Attach band linear solver 
+   * Attach band linear solver
    * ------------------------- */
 
-  flag = KINDlsSetLinearSolver(kmem, LS, J);
-  if(check_flag(&flag, "KINDlsSetLinearSolver", 1)) return(1);
+  flag = KINSetLinearSolver(kmem, LS, J);
+  if(check_flag(&flag, "KINSetLinearSolver", 1)) return(1);
 
   /* -------------------------
    * Set Jacobian function
    * ------------------------- */
 
-  flag = KINDlsSetJacFn(kmem, jac);
-  if (check_flag(&flag, "KINDlsSetJacFn", 1)) return(1);
+  flag = KINSetJacFn(kmem, jac);
+  if (check_flag(&flag, "KINSetJacFn", 1)) return(1);
 
   /* -------------
-   * Initial guess 
+   * Initial guess
    * ------------- */
 
   N_VConst_Serial(ZERO, y);
   IJth(N_VGetArrayPointer_Serial(y), 2, 2) = ONE;
 
   /* ----------------------------
-   * Call KINSol to solve problem 
+   * Call KINSol to solve problem
    * ---------------------------- */
 
   /* No scaling used */
@@ -181,7 +180,7 @@ int main()
 
 
   /* ------------------------------------
-   * Print solution and solver statistics 
+   * Print solution and solver statistics
    * ------------------------------------ */
 
   /* Get scaled norm of the system function */
@@ -193,15 +192,15 @@ int main()
   printf("\nComputed solution (||F|| = %Lg):\n\n",fnorm);
 #else
   printf("\nComputed solution (||F|| = %g):\n\n",fnorm);
-#endif  
+#endif
   PrintOutput(y);
 
   PrintFinalStats(kmem);
 
   /* -----------
-   * Free memory 
+   * Free memory
    * ----------- */
-  
+
   N_VDestroy_Serial(y);
   N_VDestroy_Serial(scale);
   KINFree(&kmem);
@@ -217,8 +216,8 @@ int main()
  *--------------------------------------------------------------------
  */
 
-/* 
- * System function 
+/*
+ * System function
  */
 
 static int func(N_Vector u, N_Vector f, void *user_data)
@@ -227,11 +226,10 @@ static int func(N_Vector u, N_Vector f, void *user_data)
   realtype hdc, vdc;
   realtype uij, udn, uup, ult, urt;
   realtype *udata, *fdata;
-  realtype x,y;
 
   int i, j;
 
-  dx = ONE/(NX+1);  
+  dx = ONE/(NX+1);
   dy = ONE/(NY+1);
   hdc = ONE/(dx*dx);
   vdc = ONE/(dy*dy);
@@ -240,13 +238,8 @@ static int func(N_Vector u, N_Vector f, void *user_data)
   fdata = N_VGetArrayPointer_Serial(f);
 
   for (j=1; j <= NY; j++) {
-
-    y = j*dy;
-
     for (i=1; i <= NX; i++) {
 
-      x = i*dx;
-
       /* Extract u at x_i, y_j and four neighboring points */
 
       uij = IJth(udata, i, j);
@@ -265,14 +258,13 @@ static int func(N_Vector u, N_Vector f, void *user_data)
       IJth(fdata, i, j) = hdiff + vdiff + uij - uij*uij*uij + 2.0;
 
     }
-
   }
 
   return(0);
 }
 
-/* 
- * Jacobian function 
+/*
+ * Jacobian function
  */
 
 static int jac(N_Vector u, N_Vector f, SUNMatrix J,
@@ -284,7 +276,7 @@ static int jac(N_Vector u, N_Vector f, SUNMatrix J,
 
   int i, j, k;
 
-  dx  = ONE/(NX+1);  
+  dx  = ONE/(NX+1);
   dy  = ONE/(NY+1);
   hdc = ONE/(dx*dx);
   vdc = ONE/(dy*dy);
@@ -320,7 +312,7 @@ static int jac(N_Vector u, N_Vector f, SUNMatrix J,
   return(0);
 }
 
-/* 
+/*
  * Print solution at selected points
  */
 
@@ -370,7 +362,7 @@ static void PrintOutput(N_Vector u)
   }
 }
 
-/* 
+/*
  * Print final statistics
  */
 
@@ -379,7 +371,7 @@ static void PrintFinalStats(void *kmem)
   long int nni, nfe, nje, nfeD;
   long int lenrwB, leniwB;
   int flag;
-  
+
   /* Main solver statistics */
 
   flag = KINGetNumNonlinSolvIters(kmem, &nni);
@@ -389,22 +381,22 @@ static void PrintFinalStats(void *kmem)
 
   /* Band linear solver statistics */
 
-  flag = KINDlsGetNumJacEvals(kmem, &nje);
-  check_flag(&flag, "KINDlsGetNumJacEvals", 1);
-  flag = KINDlsGetNumFuncEvals(kmem, &nfeD);
-  check_flag(&flag, "KINDlsGetNumFuncEvals", 1);
+  flag = KINGetNumJacEvals(kmem, &nje);
+  check_flag(&flag, "KINGetNumJacEvals", 1);
+  flag = KINGetNumLinFuncEvals(kmem, &nfeD);
+  check_flag(&flag, "KINGetNumLinFuncEvals", 1);
 
   /* Band linear solver workspace size */
 
-  flag = KINDlsGetWorkSpace(kmem, &lenrwB, &leniwB);
-  check_flag(&flag, "KINDlsGetWorkSpace", 1);
+  flag = KINGetLinWorkSpace(kmem, &lenrwB, &leniwB);
+  check_flag(&flag, "KINGetLinWorkSpace", 1);
 
   printf("\nFinal Statistics.. \n\n");
   printf("nni      = %6ld    nfe     = %6ld \n", nni, nfe);
   printf("nje      = %6ld    nfeB    = %6ld \n", nje, nfeD);
   printf("\n");
   printf("lenrwB   = %6ld    leniwB  = %6ld \n", lenrwB, leniwB);
-  
+
 }
 
 /*
@@ -414,7 +406,7 @@ static void PrintFinalStats(void *kmem)
  *    opt == 1 means SUNDIALS function returns a flag so check if
  *             flag >= 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
 static int check_flag(void *flagvalue, const char *funcname, int opt)
@@ -423,7 +415,7 @@ static int check_flag(void *flagvalue, const char *funcname, int opt)
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
+    fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
@@ -436,7 +428,7 @@ static int check_flag(void *flagvalue, const char *funcname, int opt)
       fprintf(stderr,
               "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
 	      funcname, *errflag);
-      return(1); 
+      return(1);
     }
   }
 
