@@ -1,14 +1,16 @@
 /*
- * -----------------------------------------------------------------
- * $Revision: 1.3 $
- * $Date: 2007/04/30 19:28:59 $
  * ----------------------------------------------------------------- 
  * Programmer: Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * Copyright (c) 2002, The Regents of the University of California.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * Fortran/C interface routines for CVODE, for the case of a 
  * user-supplied error weight calculation routine.
@@ -28,9 +30,9 @@
 #ifdef __cplusplus  /* wrapper to enable C++ usage */
 extern "C" {
 #endif
-  extern void FCV_EWT(realtype*, realtype*,  /* Y, EWT */ 
-                      long int*, realtype*,  /* IPAR, RPAR */
-                      int*);                 /* IER */
+  extern void FCV_EWT(realtype *Y, realtype *EWT,
+                      long int *IPAR, realtype *RPAR,
+                      int *IER);
 #ifdef __cplusplus
 }
 #endif
@@ -43,10 +45,7 @@ extern "C" {
 
 void FCV_EWTSET(int *flag, int *ier)
 {
-  CVodeMem cv_mem;
-
   if (*flag != 0) {
-    cv_mem = (CVodeMem) CV_cvodemem;
     *ier = CVodeWFtolerances(CV_cvodemem, FCVEwtSet);
   }
 }
